.MEM	8
.INDEX	8
.OPT	ON
.ORG	$C000

SPR_RAM = $0200
;
FDS4025 = $00FA

PPU2005V = $00FC
PPU2005H = $00FD
PPU2001 = $00FE
PPU2000 = $00FF
;
CONTROLLER1	.db	$00	;7-> A,B,SL,ST,U,D,L,R <-0
CONTROLLER2	.db	$00
CONT1LAST	.db	$80		;Reading of the last controller
CONT2LAST	.db	$00
CONT1STROBE	.db	$00
CONT2STROBE	.db	$00
NMI_PASS	.db	$00		;Has NMI passed?
VBCOUNT		.db	$00
UPDATEDELAY	.db	$00
MSG_FLAG	.db	$00		;See if message is to be written.
PPU_ADDR_HI	.db	$00
PPU_ADDR_LO	.db	$00

;
NMI_SUB
	php
	pha
	txa
	pha
	tya
	pha
;----------

	lda	MSG_FLAG	;Do we write any message?
	beq	+
	sec
	sbc	#$01
	jsr	Write_Msg
	lda	#$00
	sta	MSG_FLAG
+
	jsr	Controller_Magic
	jsr	BTN_READ
	jsr	$FF5C		;Music PLAY address.
	
	lda	#$01
	sta	NMI_PASS
	INC	VBCOUNT
	jsr	FDS_HOOK

	lda	PPU2005H
	sta	$2005
	lda	PPU2005V
	sta	$2005

;----------
	pla
	tay
	pla
	tax
	pla
	plp
	rti

;

Reset_SUB
	cld
	sei
	lda	#$00
	sta	$2000
	sta	PPU2000
	sta	$2001
	sta	PPU2001

-	lda	$2002		;Wait for VBlank
	bpl	-

	ldx	#$FF
	txs

	ldy	#$07
	sty	$0001
	ldy	#$00
	sty	$0000
	lda	#$00

-	sta	($0000),y	;Clear all RAM
	dey
	bne	-
	dec	$0001
	bpl	-

	jsr	INIT_PPU
	lda	#$00
	jsr	Write_Msg

	ldx	#$00
	stx	NMI_PASS

	jsr	FDS_HOOK

	lda	#$00
	sta	$2005
	sta	PPU2005H
	sta	$2005
	sta	PPU2005V
	lda	#%10001000		;NMI,Sprite CHR
	sta	$2000
	sta	PPU2000
	lda	#%00001110		;Sprites off,Screen on, no clipping.
	sta	$2001
	sta	PPU2001
	
Done_Loop
	lda	NMI_PASS
	beq	Done_Loop
	lda	$4032
	and	#$01
	beq	Done_Loop		;Disk not ejected yet.
	lda	#$02
	sta	MSG_FLAG
	lda	#$00
	sta	PPU2005V
	sta	VBCOUNT
Disk_Ejected
	lda	VBCOUNT
	cmp	#$20			;A short delay.
	bne	Disk_Ejected
-
	lda	$4032
	and	#$01			;Check if inserted.
	bne	-
Disk_Inserted				;It's inserted now.
	lda	#$03			;Ask to wait...
	sta	MSG_FLAG
	lda	#$00
	sta	PPU2005V
	sta	VBCOUNT

	lda	#$01
	sta	$E1	;Init the music!!!
-
	lda	VBCOUNT
	cmp	#$20
	bne	-

	jsr	Get_Disk_Info		;Do the whole checking process.
	
	jmp	Done_Loop
;*****************************************************************
;*****************************************************************
;------------------------------- SUBROUTINES ---------------
;

IRQ_SUB
	rti

Get_Disk_Info
	lda	#$00
	sta	$2000
	sta	PPU2000		;Turn off NMIs.
	cli			;Enable interrupts. (Dangerous?)
	jsr	$E32A		;Call the disk info routine.
	.dw	_DISK_INFO
	sei
	pha
	lda	#$2E
	sta	$4025		;To restore mirroring, dammit!
	sta	FDS4025		;The BIOS disrupts this.

	lda	#$00
	sta	$2005
	sta	PPU2005H
	sta	$2005
	sta	PPU2005V
	lda	#%10001000		;NMI,Sprite CHR
	sta	$2000
	sta	PPU2000
	pla
	beq	+		;No disk error!
	pha
	ldx	#$08
	jsr	Get_Hi_Nybble
	sta	_Message3,X
	inx
	pla
	jsr	Get_Lo_Nybble
	sta	_Message3,X		;Write the error.
	lda	#$04
	sta	MSG_FLAG
	lda	#$02
	sta	$E1			;Buubuu sound!
	lda	#$00
	sta	VBCOUNT
-
	lda	VBCOUNT
	cmp	#$80
	bne	-
	jmp	Disk_Info_End		;Branch to end.
+
	lda	#$05
	sta	MSG_FLAG
	jsr	Write_Disk_Info
Disk_Info_End
	rts

Write_Disk_Info
	lda	#$00
	sta	$2000
	sta	PPU2000
	sta	$2001
	sta	PPU2001		;Turn off screen.
	jsr	INIT_PPU
;---------------------------------------------------
	lda	#$21
	sta	$2006
	sta	PPU_ADDR_HI
	lda	#$40
	sta	$2006
	sta	PPU_ADDR_LO

	lda	#<_DISK_INFO
   	sta	$20
   	lda	#>_DISK_INFO
   	sta	$21
	jsr	Disk_Pointer	;Skip to name.

	lda	#$01
	jsr	Write_Disk_Text
	jsr	Disk_Pointer		;Writes the name.
	jsr	Write_PPU		;This is just easier.
	jsr	Disk_Pointer
	jsr	Write_PPU
	jsr	Disk_Pointer
	jsr	Write_PPU
	jsr	Disk_Pointer
	jsr	Write_PPU
	ldx	#$15
	jsr	Write_Blank

	lda	#$00
	jsr	Write_Disk_Text
	lda	_DISK_INFO		;Get Maker ID.
	pha
	jsr	Get_Hi_Nybble
	jsr	Write_PPU
	pla
	pha				;Save it again.
	jsr	Get_Lo_Nybble
	jsr	Write_PPU
	lda	#$20
	jsr	Write_PPU		;A blank
	pla
	jsr	Write_Maker_Name	;Get the proper name.
;--------------------
	lda	#$21
	sta	$2006
	sta	PPU_ADDR_HI		;Skip to next line.
	lda	#$80
	sta	$2006
	sta	PPU_ADDR_LO
;---------------------
	lda	#$02
	jsr	Write_Disk_Text		;Write Version
	jsr	Disk_Pointer		;Gets data in _DISK_INFO
	pha
	jsr	Get_Hi_Nybble
	jsr	Write_PPU
	pla
	jsr	Get_Lo_Nybble
	jsr	Write_PPU

	lda	#$03
	jsr	Write_Disk_Text		;Write Side #
	jsr	Disk_Pointer		;Gets data in _DISK_INFO
	pha
	jsr	Get_Hi_Nybble
	jsr	Write_PPU
	pla
	jsr	Get_Lo_Nybble
	jsr	Write_PPU

	lda	#$04
	jsr	Write_Disk_Text		;Write Disk#
	jsr	Disk_Pointer		;Gets data in _DISK_INFO
	pha
	jsr	Get_Hi_Nybble
	jsr	Write_PPU
	pla
	jsr	Get_Lo_Nybble
	jsr	Write_PPU

	lda	#$05
	jsr	Write_Disk_Text		;Write Extra Data
	jsr	Disk_Pointer		;Gets data in _DISK_INFO
	pha
	jsr	Get_Hi_Nybble
	jsr	Write_PPU
	pla
	jsr	Get_Lo_Nybble
	jsr	Write_PPU
	jsr	Disk_Pointer		;Gets data in _DISK_INFO
	pha
	jsr	Get_Hi_Nybble
	jsr	Write_PPU
	pla
	jsr	Get_Lo_Nybble
	jsr	Write_PPU

	lda	#$06
	jsr	Write_Disk_Text		;Write # of files
	jsr	Disk_Pointer		;Gets data in _DISK_INFO
	pha				;Push Hex #
	jsr	To_Decimal
	pha				;Push Dec #
	jsr	Get_Hi_Nybble
	jsr	Write_PPU
	pla				;Pull Dec #
	jsr	Get_Lo_Nybble
	jsr	Write_PPU
	pla				;Pull Hex #
	sta	$22			;# of files now in $22!
;-----------
	ldx	#$02
	jsr	Write_Blank
	lda	#$07
	jsr	Write_Disk_Text		;Write File Header
-
	ldx	#$03
	jsr	Write_Blank		;2 spaces
	jsr	Disk_Pointer		;Gets data in _DISK_INFO
	pha
	jsr	Get_Hi_Nybble		;File ID#
	jsr	Write_PPU
	pla
	jsr	Get_Lo_Nybble
	jsr	Write_PPU
	ldx	#$01
	jsr	Write_Blank		;1 space.

	ldy	#$08
--	jsr	Disk_Pointer		;File name.
	jsr	Write_PPU
	dey
	bne	--
	ldx	#$02
	jsr	Write_Blank		;1 more space.
	dec	$22
	bne	-

-
	lda	PPU_ADDR_LO		;Go to end of a line.
	and	#$1F			;Is it evenly-aligned?
	beq	+
	lda	#$20			;Fill with spaces.
	jsr	Write_PPU
	jmp	-
+
	ldx	#$20
	jsr	Write_Blank
	lda	#$08
	jsr	Write_Disk_Text		;Write file size...
	jsr	Disk_Pointer		;Gets data in _DISK_INFO
	pha
	jsr	Get_Hi_Nybble
	jsr	Write_PPU
	pla
	jsr	Get_Lo_Nybble
	jsr	Write_PPU
	jsr	Disk_Pointer		;Gets data in _DISK_INFO
	pha
	jsr	Get_Hi_Nybble
	jsr	Write_PPU
	pla
	jsr	Get_Lo_Nybble
	jsr	Write_PPU
;---------------------------------------------------
	lda	#$00
	sta	$2005
	sta	PPU2005H
	sta	$2005
	sta	PPU2005V
	lda	#%10001000		;NMI,Sprite CHR
	sta	$2000
	sta	PPU2000
	lda	#%00001110		;Sprites,Screen off, no clipping.
	sta	$2001
	sta	PPU2001

	rts
;_______________________________________________________

Write_Maker_Name
	sta	$00
	tya
	pha
	ldy	#$00

-
	lda	_COMPANY_INDEX,Y	;Load index of companies
	cmp	$00			;Is it equal to maker?
	beq	Matched_Name
	iny
	cpy	#$21			;Has it gone through all companies?
	bne	-			;If not, keep searching.
		;Here, it means NO match!
	ldy	#$00			;Default...

Matched_Name
	tya
	asl	;Double index
	tay
	lda	_COMPANY_POINTER,Y
	sta	$00
	iny
	lda	_COMPANY_POINTER,Y
	sta	$01
	ldy	#$00
-
	lda	($00),Y
	beq	+
	jsr	Write_PPU
	iny
	bne	-
+

	pla
	tay
	rts

Write_Disk_Text
	sta	$00
	tya
	pha
	txa
	pha
	lda	$00
	asl		;Double index
	tax
	lda	_Disk_Text_List,X
	sta	$00
	inx
	lda	_Disk_Text_List,X
	sta	$01
	ldy	#$00
-
	lda	($00),Y
	beq	+
	jsr	Write_PPU
	iny
	bne	-
+
	pla
	tax
	pla
	tay
	rts

_DT0	.db	" Maker: ",0
_DT1	.db	" Name: ",0
_DT2	.db	" Version: ",0
_DT3	.db	" Side #",0
_DT4	.db	" Disk #",0
_DT5	.db	"   Extra: ",0
_DT6	.db	"  No. of Files: ",0
;		 12345678911234567892123456789312
_DT7	.db	"                                "
	.db $81,$84,$84,$84,$84,$84,$84,$84,$84,$84,$89
	.db	"File List"
	.db $8A,$84,$84,$84,$84,$84,$84,$84,$84,$84,$84,$82
	.db	$83,"  ID Name         ID Name     ",$83
	.db $85,$84,$84,$84,$84,$84,$84,$84,$84,$84,$84
	.db $84,$84,$84,$84,$84,$84,$84,$84,$84,$84
	.db $84,$84,$84,$84,$84,$84,$84,$84,$84,$84,$86,$00
	;.db	"+------------------------------+",0
_DT8	.db	" Disk Size: $",0


_Disk_Text_List
	.dw	_DT0,_DT1,_DT2,_DT3,_DT4,_DT5,_DT6,_DT7,_DT8
;--------------------------------------------------
Write_Blank
	pha
	lda	#$20
-
	jsr	Write_PPU
	dex			;Entry: X=# of loops.
	bne	-
	pla
	rts
;----------------------------
Disk_Pointer
   	tya
   	pha

   	ldy	#$00
   	lda	($20),Y     ;Load from _DISK_INFO
	sta	$00
	inc	$20
	bne	+
	inc	$21
+
	pla
	tay
	lda	$00
	rts

;*******************************************************

Get_Hi_Nybble   
	lsr      	;Gives you the high nybble...
	lsr
	lsr
	lsr
Get_Lo_Nybble
	and	#$0F  	;Gives you the low nybble...
	clc
	adc	#$30       ;And converts to a viewable number
	cmp 	#$3A
	bcc	+	;Or HEX, if it's above 9.
	clc
	adc	#$07
+
	rts
;--------------------
To_Decimal
;--------------------------------------------
; INPUT: A: Number to convert from Hex to Dec
;--------------------------------------------
   sta   $23
   tya
   pha

   lda   $23
   ldy   #$FF
-
   iny
   sec
   sbc   #$0A
   bcs   -
   clc
   adc   #$0A
   sta   $23
   tya
   asl
   asl
   asl
   asl
   ora   $23
   sta   $23

   pla
   tay
   lda   $23
   rts
;*************************************************

Write_PPU
	sta	$2007
	pha
	inc	PPU_ADDR_LO
	bne	+		;Branch if no overflow
	inc	PPU_ADDR_HI
+
	lda	PPU_ADDR_HI
	cmp	#$23
	bne	+		;Branch if PPU != $23XX
	lda	PPU_ADDR_LO
	cmp	#$C0
	bne	+		;Branch if PPU != $23C0
	lda	#$28
	sta	$2006
	sta	PPU_ADDR_HI
	lda	#$00
	sta	$2006
	sta	PPU_ADDR_LO	;Set PPU address to $2800 now.
+
	lda	PPU_ADDR_HI
	cmp	#$2B
	bne	+		;Branch if PPU != $2BXX
	lda	PPU_ADDR_LO
	cmp	#$C0
	bne	+		;Branch if PPU != $2BC0
	lda	#$2B
	sta	$2006
	sta	PPU_ADDR_HI
	lda	#$BF
	sta	$2006
	sta	PPU_ADDR_LO	;Set PPU address to $2BBF now.
+
	pla
	rts

Write_Msg
	pha
	lda	#$20
	sta	$2006
	lda	#$E8
	sta	$2006
	pla
	asl		;Double index
	tax
	lda	_MSG_ADDR,X
	sta	$00
	inx
	lda	_MSG_ADDR,X
	sta	$01
	ldy	#$00
-
	lda	($00),Y
	beq	+
	sta	$2007
	iny
	bne	-
+
	rts
_Message0 .db "Please eject disk. ",0
_Message1 .db "Please insert disk.",0
_Message2 .db "Just a moment...   ",0
_Message3 .db "Error #$XX         ",0
_Message4 .db "OK!                ",0

_MSG_ADDR .dw _Message0,_Message1,_Message2,_Message3,_Message4

FDS_HOOK
	pha
	lda	#$C0
	sta	$0100
	lda	#$80
	sta	$0101
	lda	#$35
	sta	$0102
	lda	#$53
	sta	$0103
	pla
	rts
;

BTN_READ

	lda	CONT1STROBE
	and	#%00100000		;Check for "select"
	beq	Not_Select
	lda	PPU2000
	eor	#%00010000
	sta	$2000
	sta	PPU2000			;Toggle ASCII/FDS CHR
	
Not_Select
	lda	CONT1STROBE
	and	#%00010000
	beq	Not_Start
	
Not_Start
	lda	CONTROLLER1
	and	#%00001000	
	beq	Not_Up
	dec	PPU2005V
	dec	PPU2005V
	lda	PPU2005V
	cmp	#$FE
	bne	Not_Up
	lda	#$00
	sta	PPU2005V
	
Not_Up
	lda	CONTROLLER1
	and	#%00000100
	beq	Not_Down
	inc	PPU2005V
	inc	PPU2005V
	lda	PPU2005V
	cmp	#$F0
	bne	Not_Down
	lda	#$EE
	sta	PPU2005V

Not_Down
	lda	CONT1STROBE
	and	#$80
	beq	NO_ACTION
;
NO_ACTION
	rts

Controller_Magic
	lda	CONTROLLER1
	sta	CONT1LAST
	lda	CONTROLLER2
	sta	CONT2LAST
	ldx	#$01
	jsr	$E9EE		;Read Controller -> $F5,$F6
	lda	$F5
	sta	CONTROLLER1
	lda	$F6
	sta	CONTROLLER2
;----------------------
	lda	CONT1LAST	;Read old controller status
	eor	#$FF		;Inverse (a mask!)
	and	CONTROLLER1
	sta	CONT1STROBE	;Check strobe of controllers!
	lda	CONT2LAST
	eor	#$FF
	and	CONTROLLER2
	sta	CONT2STROBE
	rts

INIT_PPU
	
	lda	#$3f		;set up basic palette
	sta	$2006
	lda	#$00
	sta	$2006
	
	ldx	#$00
-
	lda	_Palette,X
	sta	$2007
	inx
	cpx	#$10
	bne	-

	lda	#$00
	sta	$2006
	sta	$2006

	lda   #<FONT_BIN
   	sta   $00
   	lda   #>FONT_BIN
   	sta   $01

   	ldx   #$10
   	ldy   #$00

-
   	tya
   	pha

   	ldy   #$00
   	lda   ($00),Y     ;Load up CHR File
   	sta   $2007

   	pla
   	tay

   	inc   $00
   	bne   NoDeck1
   	inc   $01

NoDeck1
   	dey
   	bne - 
   	dex
   	bne -

	lda	#$20
	sta	$2006
	lda	#$00
	sta	$2006

	ldy	#$0B
	sty	$0001
	ldy	#$00
	sty	$0000
	lda	#$00

-	sta	$2007	;Clear 2 PPU NAM tables
	dey
	bne	-
	dec	$0001
	bpl	-

	lda	#$20		;set up for title text.
	sta	$2006
;	lda	#$20
	sta	$2006
	ldx	#$00
-
	lda	_TITLE,X
	sta	$2007
	inx
	bne	-

	lda	#$23
	sta	$2006
	lda	#$C0
	sta	$2006
	ldx	#$00
-
	lda	_ATT_TABLE1,X
	sta	$2007
	inx
	cpx	#$40
	bne	-

	lda	#$2B
	sta	$2006
	lda	#$C0
	sta	$2006
	ldx	#$00
	lda	#$FF
-
	sta	$2007
	inx
	cpx	#$40
	bne	-
	rts


_Palette .db $0F,$17,$27,$38,$0F,$00,$10,$20,$0F,$15,$25,$35,$0F,$11,$21,$31
;--------------------------------------------------------------------------

FONT_BIN	.INCBIN "FONT4k.CHR"
_TITLE		.INCBIN "TITLE.BIN"
_ATT_TABLE1	.INCBIN "ATTTBL1.BIN"

.INCLUDE "Makers.ASM"

_DISK_INFO
	.dd $FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF
	.dd $FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF
	.dd $FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF
	.dd $FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF
	.dd $FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF
	.dd $FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF
	.dd $FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF
	.dd $FFFFFFFF,$FFFFFFFF,$FFFFFFFF,$FFFFFFFF
	;Disk info area (don't put anything after it!)

.PAD $DFF6
	.dw NMI_SUB
	.dw NMI_SUB
	.dw NMI_SUB
	.dw Reset_SUB
	.dw IRQ_SUB